@@ -45,6 +45,7 @@
 
 #include <libgnome/libgnome.h>
 #include <bonobo/bonobo-ui-main.h>
+#include <gconf/gconf-client.h>
 
 #include "gnome-client.h"
 #include "gnome-gconf-ui.h"
@@ -92,6 +93,14 @@ static void libgnomeui_segv_setup	(GnomeProgram *program,
 /* Prototype for a private gnome_stock function */
 void _gnome_stock_icons_init (void);
 
+/* Whether to make noises when the user clicks a button, etc.  We cache it
+ * in a boolean rather than querying GConf every time.
+ */
+static gboolean use_event_sounds;
+
+/* GConf client for monitoring the event sounds option */
+static GConfClient *gconf_client = NULL;
+
 
 enum { ARG_DISABLE_CRASH_DIALOG=1, ARG_DISPLAY };
 
@@ -283,12 +292,15 @@ relay_gtk_signal(GSignalInvocationHint *hint,
 		 const GValue *param_values,
 		 gchar *signame)
 {
-  char *pieces[3] = {"gtk-events", NULL, NULL};
+  char *pieces[3] = {"gtk-events-2", NULL, NULL};
   static GQuark disable_sound_quark = 0;
   GObject *object = g_value_get_object (&param_values[0]);
 
   pieces[1] = signame;
 
+  if (!use_event_sounds)
+    return TRUE;
+
   if(!disable_sound_quark)
     disable_sound_quark = g_quark_from_static_string("gnome_disable_sound_events");
 
@@ -303,25 +315,26 @@ relay_gtk_signal(GSignalInvocationHint *hint,
       return TRUE;
   }
 
+  if (gnome_sound_connection_get() < 0)
+    return TRUE;
+
   gnome_triggers_vdo("", NULL, (const char **)pieces);
 
   return TRUE;
 }
-#endif
 
 static void
 initialize_gtk_signal_relay (void)
 {
-#ifdef HAVE_ESD
 	gpointer iter_signames;
 	char *signame;
 	char *ctmp, *ctmp2;
-	
-	if (gnome_sound_connection_get () < 0)
-		return;
-	
-	if (!gnome_gconf_get_bool ("/desktop/gnome/sound/event_sounds"))
-		return;
+        static gboolean initialized = FALSE;
+
+        if (initialized)
+                return;
+
+        initialized = TRUE;
 	
 	ctmp = gnome_config_file ("/sound/events/gtk-events-2.soundlist");
 	ctmp2 = g_strconcat ("=", ctmp, "=", NULL);
@@ -341,14 +354,14 @@ initialize_gtk_signal_relay (void)
 		 * knowledge of what gtk widgets do what, rather than
 		 * anything based on the info available at runtime.
 		 */
-		if(!strcmp (signame, "activate")){
+		if (!strcmp (signame, "activate")) {
 			g_type_class_unref (g_type_class_ref (gtk_menu_item_get_type ()));
 			signums [0] = g_signal_lookup (signame, gtk_menu_item_get_type ());
 			
 			g_type_class_unref (g_type_class_ref (gtk_entry_get_type ()));
 			signums [1] = g_signal_lookup (signame, gtk_entry_get_type ());
 			nsigs = 2;
-		} else if(!strcmp(signame, "toggled")){
+		} else if (!strcmp(signame, "toggled")) {
 			g_type_class_unref (g_type_class_ref (gtk_toggle_button_get_type ()));
 			signums [0] = g_signal_lookup (signame,
 							 gtk_toggle_button_get_type ());
@@ -357,7 +370,7 @@ initialize_gtk_signal_relay (void)
 			signums [1] = g_signal_lookup (signame,
 							 gtk_check_menu_item_get_type ());
 			nsigs = 2;
-		} else if (!strcmp (signame, "clicked")){
+		} else if (!strcmp (signame, "clicked")) {
 			g_type_class_unref (g_type_class_ref (gtk_button_get_type ()));
 			signums [0] = g_signal_lookup (signame, gtk_button_get_type ());
 			nsigs = 1;
@@ -368,19 +381,61 @@ initialize_gtk_signal_relay (void)
 		}
 
 		used_signame = FALSE;
-		for(i = 0; i < nsigs; i++) {
+		for (i = 0; i < nsigs; i++) {
 			if (signums [i] > 0) {
-				g_signal_add_emission_hook (signums [i], 0,
-							      (GSignalEmissionHook)relay_gtk_signal,
-							      signame, NULL);
+				g_signal_add_emission_hook (
+                                        signums [i], 0,
+                                        (GSignalEmissionHook) relay_gtk_signal,
+                                        signame, NULL);
                                 used_signame = TRUE;
                         }
                 }
 
-                if(!used_signame)
-                        g_free(signame);
+                if (!used_signame)
+                        g_free (signame);
 	}
 	gnome_config_pop_prefix ();
+}
+
+/* Callback used when the GConf event_sounds key's value changes */
+static void
+event_sounds_changed_cb (GConfClient* client, guint cnxn_id, GConfEntry *entry, gpointer data)
+{
+        gboolean new_use_event_sounds;
+
+        new_use_event_sounds = (gnome_gconf_get_bool ("/desktop/gnome/sound/enable_esd") &&
+                                gnome_gconf_get_bool ("/desktop/gnome/sound/event_sounds"));
+
+        if (new_use_event_sounds && !use_event_sounds)
+                initialize_gtk_signal_relay ();
+
+        use_event_sounds = new_use_event_sounds;
+}
+#endif
+
+static void
+setup_event_listener (void)
+{
+#ifdef HAVE_ESD
+        gconf_client = gconf_client_get_default ();
+        if (!gconf_client)
+                return;
+
+        gconf_client_add_dir (gconf_client, "/desktop/gnome/sound",
+                              GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
+        gconf_client_notify_add (gconf_client, "/desktop/gnome/sound/event_sounds",
+                                 event_sounds_changed_cb,
+                                 NULL, NULL, NULL);
+        gconf_client_notify_add (gconf_client, "/desktop/gnome/sound/enable_esd",
+                                 event_sounds_changed_cb,
+                                 NULL, NULL, NULL);
+	
+        use_event_sounds = (gnome_gconf_get_bool ("/desktop/gnome/sound/enable_esd") &&
+                            gnome_gconf_get_bool ("/desktop/gnome/sound/event_sounds"));
+
+        if (use_event_sounds)
+                initialize_gtk_signal_relay ();
 #endif
 }
 
@@ -412,7 +467,7 @@ libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
 
         _gnome_stock_icons_init ();
 
-	initialize_gtk_signal_relay ();
+        setup_event_listener ();
 }
 
 static void
@@ -456,7 +511,7 @@ libgnomeui_arg_callback(poptContext con,
  * It provides good consistancy (you always know the rc file will be
  * the same name as the executable), and it's easy for the programmer.
  * 
- * If you don't like it.. give me a good reason.  Symlin
+ * If you don't like it.. give me a good reason.
  */
 static void
 libgnomeui_rc_parse (GnomeProgram *program, const gchar *command)
@@ -605,7 +660,7 @@ static void libgnomeui_segv_handle(int signum)
                 program = gnome_program_get();
 
 		/* Child process */
-		execl (GNOMEUIBINDIR "/gnome_segv2", GNOMEUIBINDIR "/gnome_segv",
+		execl (GNOMEUISERVERDIR "/gnome_segv2", GNOMEUISERVERDIR "/gnome_segv",
 		       g_get_prgname (), buf,
 		       gnome_program_get_app_version (program), NULL);
 
@@ -634,13 +689,15 @@ static void libgnomeui_segv_handle(int signum)
  *
  * Initializes the application.  This sets up all of the GNOME
  * internals and prepares them (imlib, gdk, session-management, triggers,
- * sound, user preferences)
+ * sound, user preferences).
  *
  * Unlike #gnome_init, with #gnome_init_with_popt_table you can provide
  * a table of popt options (popt is the command line argument parsing
  * library).
  *
- * Deprecated, use #gnome_program_init with the LIBGNOMEUI_MODULE
+ * Deprecated, use #gnome_program_init with the LIBGNOMEUI_MODULE.
+ *
+ * Returns: 0 (always)
  */
 int
 gnome_init_with_popt_table (const char *app_id,
